/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

// Semaphore.cpp: implementation of the Semaphore class.
//
//////////////////////////////////////////////////////////////////////

#include "Semaphore.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Semaphore::Semaphore()
{
	IsAvailable = true;
	LocksCount = 0;
}

Semaphore::~Semaphore()
{
}


void Semaphore::Wait() const
{
	AccessLock.EnterCS();
	if(!IsAvailable)
	{
		//Am I the owner ?
		if(OwnerThread == NewpkiThread::CurrentThreadId())
		{
			((Semaphore*)this)->LocksCount++;
			AccessLock.LeaveCS();
			return;
		}
		else
		{
			//Wait for resource to be available
			do
			{
				AccessLock.LeaveCS();
				NewpkiThread::Sleep(50);
				AccessLock.EnterCS();
			}
			while(!IsAvailable);
		}
	}

	((Semaphore*)this)->LocksCount++;
	((Semaphore*)this)->IsAvailable = false;
	((Semaphore*)this)->OwnerThread = NewpkiThread::CurrentThreadId();

	AccessLock.LeaveCS();
}

void Semaphore::Post() const
{
	AccessLock.EnterCS();
	((Semaphore*)this)->LocksCount--;

	assert(LocksCount>=0);
	if(LocksCount<=0)
	{
		((Semaphore*)this)->LocksCount = 0;
		((Semaphore*)this)->IsAvailable = true;
	}
	AccessLock.LeaveCS();
}


bool Semaphore::isOwner(NEWPKI_THREAD_ID id) const
{
	return (OwnerThread == id);
}

void Semaphore::setOwner(NEWPKI_THREAD_ID id) const
{
	((Semaphore*)this)->OwnerThread = id;
}
